"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const dotenv_1 = __importDefault(require("dotenv"));
const web3_1 = __importDefault(require("web3"));
require("mocha");
const moment_1 = __importDefault(require("moment"));
const origin_backend_client_mocks_1 = require("@energyweb/origin-backend-client-mocks");
const migrate_1 = require("../migrate");
const __1 = require("..");
const Logger_1 = require("../Logger");
describe('Cerificate tests', () => {
    let conf;
    let publicIssuer;
    let privateIssuer;
    dotenv_1.default.config({
        path: '.env.test'
    });
    const web3 = new web3_1.default(process.env.WEB3);
    const deployKey = process.env.DEPLOY_KEY;
    const privateKeyDeployment = deployKey.startsWith('0x') ? deployKey : `0x${deployKey}`;
    const accountDeployment = web3.eth.accounts.privateKeyToAccount(privateKeyDeployment).address;
    const deviceOwnerPK = '0x622d56ab7f0e75ac133722cc065260a2792bf30ea3265415fe04f3a2dba7e1ac';
    const accountDeviceOwner = web3.eth.accounts.privateKeyToAccount(deviceOwnerPK).address;
    const issuerPK = '0x50397ee7580b44c966c3975f561efb7b58a54febedaa68a5dc482e52fb696ae7';
    const issuerAccount = web3.eth.accounts.privateKeyToAccount(issuerPK).address;
    const traderPK = '0xca77c9b06fde68bcbcc09f603c958620613f4be79f3abb4b2032131d0229462e';
    const accountTrader = web3.eth.accounts.privateKeyToAccount(traderPK).address;
    const setActiveUser = (privateKey) => {
        conf.blockchainProperties.activeUser = {
            address: web3.eth.accounts.privateKeyToAccount(privateKey).address,
            privateKey
        };
    };
    const issueCertificate = async (volume, isPrivate = false) => {
        setActiveUser(issuerPK);
        const now = moment_1.default();
        const generationStartTime = now.subtract(30, 'day').unix();
        const generationEndTime = now.unix();
        const deviceId = '1';
        return __1.Certificate.createCertificate(accountDeviceOwner, volume, generationStartTime, generationEndTime, deviceId, conf, isPrivate);
    };
    it('migrates Registry', async () => {
        const registry = await migrate_1.migrateRegistry(web3, privateKeyDeployment);
        publicIssuer = await migrate_1.migratePublicIssuer(web3, issuerPK, registry.web3Contract.options.address);
        privateIssuer = await migrate_1.migratePrivateIssuer(web3, issuerPK, registry.web3Contract.options.address, publicIssuer.web3Contract.options.address);
        conf = {
            blockchainProperties: {
                activeUser: {
                    address: accountDeployment,
                    privateKey: privateKeyDeployment
                },
                certificateLogicInstance: registry,
                issuerLogicInstance: {
                    public: publicIssuer,
                    private: privateIssuer
                },
                web3
            },
            offChainDataSource: new origin_backend_client_mocks_1.OffChainDataSourceMock(),
            logger: Logger_1.logger
        };
    });
    it('issuer issues a certificate', async () => {
        const volume = 1e9;
        const certificate = await issueCertificate(volume);
        chai_1.assert.isNotNull(certificate.id);
        setActiveUser(issuerPK);
        chai_1.assert.isFalse(await certificate.isOwned());
        setActiveUser(deviceOwnerPK);
        chai_1.assert.isTrue(await certificate.isOwned());
        const ownedVolume = await certificate.ownedVolume();
        chai_1.assert.equal(ownedVolume, volume);
        chai_1.assert.deepOwnInclude(certificate, {
            initialized: true,
            issuer: publicIssuer.web3Contract.options.address
        });
    });
    it('transfers a certificate', async () => {
        const totalVolume = 1e9;
        const certificate = await issueCertificate(totalVolume);
        setActiveUser(deviceOwnerPK);
        await certificate.transfer(accountTrader, totalVolume / 4);
        chai_1.assert.isTrue(await certificate.isOwned());
        chai_1.assert.equal(await certificate.ownedVolume(), (totalVolume / 4) * 3);
        setActiveUser(traderPK);
        chai_1.assert.isTrue(await certificate.isOwned());
        chai_1.assert.equal(await certificate.ownedVolume(), totalVolume / 4);
    });
    it('fails claiming a revoked certificate', async () => {
        const totalVolume = 1e9;
        const certificate = await issueCertificate(totalVolume);
        setActiveUser(issuerPK);
        await certificate.revoke();
        setActiveUser(deviceOwnerPK);
        let failed = false;
        try {
            await certificate.claim(totalVolume);
        }
        catch (e) {
            failed = true;
        }
        chai_1.assert.isTrue(failed);
    });
    it('claims a certificate', async () => {
        const totalVolume = 1e9;
        const certificate = await issueCertificate(totalVolume);
        setActiveUser(deviceOwnerPK);
        const amountToSendToTrader = totalVolume / 4;
        await certificate.transfer(accountTrader, amountToSendToTrader);
        setActiveUser(traderPK);
        await certificate.claim(amountToSendToTrader);
        chai_1.assert.isFalse(await certificate.isOwned());
        chai_1.assert.equal(await certificate.ownedVolume(), 0);
        chai_1.assert.isTrue(await certificate.isClaimed());
        chai_1.assert.equal(await certificate.claimedVolume(), amountToSendToTrader);
    });
    it('batch transfers certificates', async () => {
        const totalVolume = 1e9;
        let certificate = await issueCertificate(totalVolume);
        let certificate2 = await issueCertificate(totalVolume);
        setActiveUser(deviceOwnerPK);
        chai_1.assert.isTrue(await certificate.isOwned());
        chai_1.assert.isTrue(await certificate2.isOwned());
        chai_1.assert.equal(await certificate.ownedVolume(), totalVolume);
        chai_1.assert.equal(await certificate2.ownedVolume(), totalVolume);
        await __1.Certificate.transferCertificates([certificate.id, certificate2.id], accountTrader, conf);
        certificate = await certificate.sync();
        certificate2 = await certificate2.sync();
        chai_1.assert.isFalse(await certificate.isOwned());
        chai_1.assert.isFalse(await certificate2.isOwned());
        chai_1.assert.equal(await certificate.ownedVolume(), 0);
        chai_1.assert.equal(await certificate2.ownedVolume(), 0);
        setActiveUser(traderPK);
        chai_1.assert.isTrue(await certificate.isOwned());
        chai_1.assert.isTrue(await certificate2.isOwned());
        chai_1.assert.equal(await certificate.ownedVolume(), totalVolume);
        chai_1.assert.equal(await certificate2.ownedVolume(), totalVolume);
    });
    it('batch claims certificates', async () => {
        const totalVolume = 1e9;
        let certificate = await issueCertificate(totalVolume);
        let certificate2 = await issueCertificate(totalVolume);
        setActiveUser(deviceOwnerPK);
        chai_1.assert.isFalse(await certificate.isClaimed());
        chai_1.assert.isFalse(await certificate2.isClaimed());
        chai_1.assert.equal(await certificate.claimedVolume(), 0);
        chai_1.assert.equal(await certificate2.claimedVolume(), 0);
        await __1.Certificate.claimCertificates([certificate.id, certificate2.id], conf);
        certificate = await certificate.sync();
        certificate2 = await certificate2.sync();
        chai_1.assert.isTrue(await certificate.isClaimed());
        chai_1.assert.isTrue(await certificate2.isClaimed());
        chai_1.assert.equal(await certificate.claimedVolume(), totalVolume);
        chai_1.assert.equal(await certificate2.claimedVolume(), totalVolume);
    });
});
//# sourceMappingURL=Certificate.test.js.map