"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const dotenv_1 = __importDefault(require("dotenv"));
const web3_1 = __importDefault(require("web3"));
require("mocha");
const moment_1 = __importDefault(require("moment"));
const origin_backend_client_mocks_1 = require("@energyweb/origin-backend-client-mocks");
const migrate_1 = require("../migrate");
const __1 = require("..");
const Logger_1 = require("../Logger");
describe('PrivateIssuer', () => {
    let publicIssuer;
    let privateIssuer;
    let registry;
    let conf;
    dotenv_1.default.config({
        path: '.env.test'
    });
    const web3 = new web3_1.default(process.env.WEB3);
    const deployKey = process.env.DEPLOY_KEY;
    const privateKeyDeployment = deployKey.startsWith('0x') ? deployKey : `0x${deployKey}`;
    const accountDeployment = web3.eth.accounts.privateKeyToAccount(privateKeyDeployment).address;
    const deviceOwnerPK = '0xd9bc30dc17023fbb68fe3002e0ff9107b241544fd6d60863081c55e383f1b5a3';
    const accountDeviceOwner = web3.eth.accounts.privateKeyToAccount(deviceOwnerPK).address;
    const issuerPK = '0x50397ee7580b44c966c3975f561efb7b58a54febedaa68a5dc482e52fb696ae7';
    const setActiveUser = (privateKey) => {
        conf.blockchainProperties.activeUser = {
            address: web3.eth.accounts.privateKeyToAccount(privateKey).address,
            privateKey
        };
    };
    const createRequestIssue = async (conf) => {
        setActiveUser(deviceOwnerPK);
        const now = moment_1.default();
        const fromTime = now.subtract(30, 'day').unix();
        const toTime = now.unix();
        const deviceId = '1';
        return __1.RequestIssue.createRequestIssue(fromTime, toTime, deviceId, conf, true);
    };
    it('migrates PrivateIssuer and Registry', async () => {
        registry = await migrate_1.migrateRegistry(web3, privateKeyDeployment);
        publicIssuer = await migrate_1.migratePublicIssuer(web3, privateKeyDeployment, registry.web3Contract.options.address);
        privateIssuer = await migrate_1.migratePrivateIssuer(web3, privateKeyDeployment, registry.web3Contract.options.address, publicIssuer.web3Contract.options.address);
        const version = await privateIssuer.version();
        chai_1.assert.equal(version, 'v0.1');
        const registryAddress = await privateIssuer.getRegistryAddress();
        chai_1.assert.equal(registryAddress, registry.web3Contract.options.address);
        const publicIssuerAddress = await privateIssuer.getPublicIssuerAddress();
        chai_1.assert.equal(publicIssuerAddress, publicIssuer.web3Contract.options.address);
        conf = {
            blockchainProperties: {
                activeUser: {
                    address: accountDeployment,
                    privateKey: privateKeyDeployment
                },
                issuerLogicInstance: {
                    public: publicIssuer,
                    private: privateIssuer
                },
                web3
            },
            offChainDataSource: new origin_backend_client_mocks_1.OffChainDataSourceMock(),
            logger: Logger_1.logger
        };
    });
    it('user correctly requests issuance', async () => {
        const requestIssue = await createRequestIssue(conf);
        chai_1.assert.isAbove(Number(requestIssue.id), -1);
        chai_1.assert.deepOwnInclude(requestIssue, {
            initialized: true,
            deviceId: '1',
            owner: accountDeviceOwner,
            approved: false,
            isPrivate: true
        });
    });
    it('issuer correctly approves issuance', async () => {
        let requestIssue = await createRequestIssue(conf);
        setActiveUser(issuerPK);
        const volume = 1000;
        const certificateId = await requestIssue.approve(accountDeviceOwner, volume);
        requestIssue = await requestIssue.sync();
        chai_1.assert.isTrue(requestIssue.approved);
        const deviceOwnerBalance = await registry.balanceOf(accountDeviceOwner, Number(certificateId));
        chai_1.assert.equal(deviceOwnerBalance, 0);
    });
});
//# sourceMappingURL=PrivateIssuer.test.js.map